#include "resultset.h"

/**
 * 
 * @param sql SQL
 * @return SQLITE G[R[h
 */
bool
ResultSet::open(const wchar_t *sql)
{
	close();
	if (sql) {
		int ret = sqlite3_prepare16(db, sql, -1, &stmt, NULL);
		if (ret == SQLITE_OK) {
			reset();
		} else {
			//  XXX G[\H
		}
		return ret == SQLITE_OK;
	}
	return false;
}

/**
 * ̍s擾
 * @return ǂ
 */
bool
ResultSet::next()
{
	if (stmt) {
		if (sqlite3_step(stmt) == SQLITE_ROW) {
			return true;
		}
		reset();
	} 
	return false;
}

/**
 * Zbg
 */
void
ResultSet::reset()
{
	sqlite3_reset(stmt);
	posCount = 1;
}

/**
 * 
 */
void
ResultSet::close()
{
	if (stmt) {
		sqlite3_finalize(stmt);
		stmt = NULL;
	}
}

/**
 * G[R[h̎擾
 */
int
ResultSet::getErrorCode()
{
	return sqlite3_errcode(db);
}

/**
 * G[bZ[W̎擾
 */
const wchar_t *
ResultSet::getErrorMessage()
{
	return (const wchar_t *)sqlite3_errmsg16(db);
}

/**
 * JJԍ擾
 * @param column J
 * @return Jԍ
 */
int
ResultSet::getColumnNoFromName(const wchar_t *column)
{
	if (stmt) {
		for (int i=0; i<sqlite3_column_count(stmt); i++) {
			if (wcscmp(column, (const wchar_t *)sqlite3_column_name16(stmt, i)) == 0)  {
				return i;
			}
		}
	}
	return 0;
}

/**
 * loCh
 * @param variant l
 * @param pos ꏊw
 */
void
ResultSet::bind(tTJSVariant &variant, int pos)
{
	if (stmt) {
		if (pos) {posCount = pos;}
		switch (variant.Type()) {
		case tvtVoid:
		case tvtObject:
			sqlite3_bind_null(stmt, posCount++);
			break;
		case tvtString:
			{
				sqlite3_bind_text16(stmt, posCount++, variant.GetString(), variant.AsStringNoAddRef()->GetLength()*2, SQLITE_TRANSIENT);
			}
			break;
		case tvtOctet:
			{
				tTJSVariantOctet *octet = variant.AsOctetNoAddRef();
				sqlite3_bind_blob(stmt, posCount++, octet->GetData(), octet->GetLength(), SQLITE_STATIC);
			}
			break;
		case tvtInteger:
			sqlite3_bind_int(stmt, posCount++, (int)variant);
			break;
		case tvtReal:
			sqlite3_bind_double(stmt, posCount++, (double)variant);
			break;
		}
	}
}

int
ResultSet::getCount()
{
	return stmt ? sqlite3_column_count(stmt) : 0;
}

bool
ResultSet::isNull(int num)
{
	return stmt && sqlite3_column_type(stmt, num) == SQLITE_NULL;
}

/**
 * J̌^擾
 */
int
ResultSet::getType(int num)
{
	if (stmt) {
		switch (sqlite3_column_type(stmt, num)) {
		case SQLITE_INTEGER:
			return tvtInteger;
		case SQLITE_FLOAT:
			return tvtReal;
		case SQLITE_TEXT:
			return tvtString;
		case SQLITE_BLOB:
			return tvtOctet;
		}
	}
	return tvtVoid;
}

/**
 * w肵J̃f[^擾
 * @param num Jԍ
 * @return 擾
 */
tTJSVariant *
ResultSet::get(int num)
{
	if (stmt) {
		switch (sqlite3_column_type(stmt, num)) {
		case SQLITE_INTEGER:
			return new tTJSVariant((tjs_int32)sqlite3_column_int(stmt, num));
		case SQLITE_FLOAT:
			return new tTJSVariant(sqlite3_column_double(stmt, num));
		case SQLITE_TEXT:
			return new tTJSVariant((const wchar_t *)sqlite3_column_text16(stmt, num));
		case SQLITE_BLOB:
			return new tTJSVariant((const tjs_uint8 *)sqlite3_column_blob(stmt, num), sqlite3_column_bytes(stmt, num));
		}
	}
	return new tTJSVariant();
}

/**
 * w肵J̃f[^擾
 * @param variant ʊi[
 * @param num Jԍ
 * @param defaultVariant ftHgl
 */
void
ResultSet::get(tTJSVariant *result, int num, tTJSVariant *defaultVariant)
{
	if (stmt) {
		switch (sqlite3_column_type(stmt, num)) {
		case SQLITE_INTEGER:
			*result = (tjs_int32)sqlite3_column_int(stmt, num);
			break;
		case SQLITE_FLOAT:
			*result = sqlite3_column_double(stmt, num);
			break;
		case SQLITE_TEXT:
			*result =(const wchar_t *)sqlite3_column_text16(stmt, num);
			break;
		case SQLITE_BLOB:
			*result = tTJSVariant((const tjs_uint8 *) sqlite3_column_blob(stmt, num), sqlite3_column_bytes(stmt, num));
			break;
		}
	}
	if (defaultVariant) {
		*result = *defaultVariant;
	}
}
